/*
 * A RECEPIE FOR  LEO - Light Emitting Object
 *               -----------------------------
 * 
 * Open source modular lamp project by, Dimitri Leimgrübler, Nicolas Lienhard and Elio Amato.
 * Realised in the workshop week, spring semester 2017.
 * Workshop professors are MASAMICHI UDAGAWA and SIGI MOESLINGER from Antenna Design.
 * 
 * Fachhochschule Nordwestschweiz FHNW
 * Hochschule für Gestaltung und Kunst
 * Institut Industrial Design
 * Freilager-Platz 1
 * CH-4023 Basel
 * 
 * code by Dimitri Leimgrübler
 * 
 * 
 * Parts needed for the electronics:
 * 
 * 10W LED Module
 * 10W LED Driver Module for 12V
 * IRF520 Power MOSFET
 * 1MOhm Resistor
 * 5V Pro Mini Arduino compatible board
 * Soldering Iron
 * Multimeter for continuity testing
 * 
 * 
 * v1.0
 */





#include <CapacitiveSensor.h>   // importing this library adds the capacitive touch control functions
                                

// ##### creating neccesary variables: ####

// variables to control the output
int brightness = 255;           // set brighness variable, min 0 - 255 max
const int min_brightness = 5;   // the minimum brightness for the output, min 0 - 255 max
boolean lightOn = false;        // varable to activate the output, false - true

// variables for the touch interface
const int threshold = 1000;     // treshold value for touch queries, this may be calibrated
boolean touching = false;       // stores the state of touching in the CURRENT code cycle, false - true
boolean prev_touching = false;  // stores the state of touching in the PREVIOUS code cycle, false - true

// timing & dimming stuff
int timer;                      // a counter to build time based functions, this is an emty variable
const int dim_delay = 50;       // after this amount of time, the dimming mode sets in, number of code cycles
boolean rising = false;         // sets the dimming mode to brighter or dimmer, false - true
boolean dimming = false;        // varable to let the program know when its in dimming mode or not, false - true

// set the output pin
const int led = 11;             // MOSFET output pin, connects directly to MOSFET Gate pin
                                // pin 11 supports PWM wich is neccesary for dimming (it goes to eleven)


// set up the touch interface
CapacitiveSensor capSensor = CapacitiveSensor(4, 2);    // define the touch interface pins
                                                        // pin 4 with 1 Megaohm to electrode
                                                        // pin 2 directly to electrode



// #### functions: #####

void setup() {                  // everything in the setup function is executed once at the initial boot of the arduino
  
  pinMode(led, OUTPUT);         // set "led" pin to be handled as an output
  pinMode(LED_BUILTIN, OUTPUT); // we are using the pro minis onboard led to indicate if the plate is touched or not,
                                // so the corresponding pin also as an output

}                               // end of setup function

void loop() {                   // code inside the loop function is repeated over and over again

  // #### sensor part: ######
  
  long sensorValue = capSensor.capacitiveSensor(30);  // read the value from the sensor interface and store it in a variable

  if (sensorValue > threshold){           // If the sensor value is higher than the set treshold, 
    touching = true;                      // the plate is touched. The indicator LED turns on.
    digitalWrite(LED_BUILTIN, HIGH);
  } else {                                // Else, the plate is not touched, and the indicator LED goes off.
    touching = false;
    digitalWrite(LED_BUILTIN, LOW);
  }

  // #### dimming part: #####                             // This are the functions wich handle dimming

  if (touching == true && timer <= dim_delay){            // If the plate is touched and the timer varable is smaller than the set delay,
    timer++;                                              // the timer starts to count. Each code cycle its value increases by one.
  } else if (touching == true && timer >= dim_delay) {    // If the plate is still touched and the timer variable is BIGGER than the set delay,
    dimming = true;                                       // the lamp is going to dimming mode.
    lightOn = true;                                       // The lamp turns it self on in dimming mode, so you can enter dimming mode, even the lamp was previously off.
    if(rising){                                           // If the rising mode is ON, the lamp gets brighter.
      brightness++;                                       // increasing the brightness variable one unit at a code cycle
      if(brightness > 255){                               // the output function that we use only handles values form 0 - 255
        brightness = 255;                                 // so if the brighness value exceeds 255 it geths thrown back to 255
      }
    } else {                                              // same thing when the rising mode is OFF
      brightness--;                                       // but the brightness value decreases one unit at a code cycle
      if(brightness < min_brightness){                    // If the brightness values falls below the set minimum brightess,
        brightness = min_brightness;                      // the value is set to the minimum brighness. This prevents turning the lamp off in dimming mode.
      }
    }    
  }

  if (touching == false && prev_touching == true && dimming == true){         // If the lamp is in dimming mode and you release your finger from the touch plate,
    rising = !rising;                                                         // the rising variable inverts
    dimming = false;                                                          // and the lamp exits dimming mode.
    timer = 0;                                                                // The timer is reset to 0.
  }

  

  // #### switching part: #####                                               // This is the part where the lamp is simply switched on or off.

  else if (touching == false && prev_touching == true && timer < dim_delay){  // ELse, if you touch the plate and release your finger without exceeding the set timespan,
    lightOn = !lightOn;                                                       // the lamp is simpy toggled on or off.
    timer = 0;                                                                // The timer is reset to 0.
  }

  prev_touching = touching;                                                    // Here we store the current touch state into the "prev_touching" variable.
                                                                               // This is important, because the "touching" variable gets overwritten
                                                                               // at the beginning of each code cycle.
                                                                               // The "prev_touching" variable is needed to detect the releasing 
                                                                               // of the finger from the touch plate.
                                                                               
  // #### output part ######                               // enough of playing with varables, now we turn the physical light on and off

  if(lightOn){                                             // if the "lightOn" variable is set to true,
    analogWrite(led, brightness);                          // the "led" pin sends out a PWM signal to the gate pin of the transistor.
  } else {                                                 // else the gate of the transistor is pulled to ground (LOW), preventing the LED to recieve any current.
    digitalWrite(led, LOW);
  }

}                                                          // end of the loop functions, the code jumps back to the beginning.
